package RPM::Source::Convert::ALTLinuxImport;
use strict;
use warnings;
use Carp;
use RPM qw/vercmp evrcmp/;
use RPM::Source::Convert;
use RPM::Source::Transformation::Factory::RaiseRelease;
use RPM::Source::Convert::Plugins::Generic2ALT;

$RPM::Source::Transform::cleanup=0;
$RPM::Source::Transformation::Factory::RaiseRelease::nextrel='srpmimport';
$RPM::Source::Transformation::Factory::RaiseRelease::defaultchangelog='- converted for ALT Linux by srpmconvert tools';
&RPM::Source::Transformation::Factory::RaiseRelease::register_nextrel_method(
    -name => 'srpmimport',
    -nextrel_method => \&_nextrel_srpmimport,
    -initrel_method => \&_initrel_srpmimport,
    );

sub __cleanup_foreign_release {
    my $rel=shift;
    # for fedora-based imports (hack :()
    $rel=~s/\%\{\??dist\}//;
    # mageia specific
    $rel=~s/%mkrel\s+//;
    $rel=~s/^\s+//;
    return $rel;
}

sub _initrel_srpmimport {
    my ($spec,$parent)=@_;
    my $rel = $spec->get_section('package','')->get_tag('Release');
    return 'alt1_'.&__cleanup_foreign_release($rel);
}

sub _nextrel_srpmimport {
    my ($rel,$spec,$parent)=@_;
    my $origver = $spec->get_section('package','')->get_tag('Version');
    print STDERR "_nextrel_srpmimport: input rel=$rel\n" if $RPM::Source::Transform::verbose>1;
    $rel = &__cleanup_foreign_release($rel);
    my $altrel="alt1";
    if ($parent) {
	$altrel = $parent->get_section('package','')->get_tag('Release');
	my $altver = $parent->get_section('package','')->get_tag('Version');
	my $vercmp = evrcmp($altver,$origver);
	if ($altrel=~/^alt([^_]+)_(.+)$/) {
	    my $relcore = $1;
	    my $origrel = $2;
	    # hack -- cleanup (undefined in jpp) macro in alt release:
	    $relcore=~s/.?\%\w+//g;
	    $relcore=~s/.?\%\{\w+\}+//g;
	    print STDERR "_nextrel_srpmimport: rel=$rel relcore=$relcore origrel=$origrel\n" if $RPM::Source::Transform::verbose>1;
	    if ($relcore=~/^0\./ or $vercmp != 0) {
		$relcore=1;
	    } elsif (evrcmp($origrel,$rel) >= 0 ) { ## also when $origrel eq $rel
		$relcore=~/^(\d+)/;
		$relcore=$1+1;
	    }
	    $altrel='alt'.$relcore;
	} else {
	    $altrel=~s/.?\%\w+//g;
	    $altrel=~s/.?\%\{\w+\}+//g;
	    $altrel='alt1' if $altrel=~/^alt0\./ or $vercmp != 0;
	}
    }
    
    my $newrel = $altrel."_".$rel;
    return $newrel;
}

1;
