package RPM::Source::Convert::Plugins::Fedora2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Convert::Plugins::Generic2ALT;
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
# after Generic2ALT; not to disturb the order
use RPM::Source::Transformation::Factory::DependencyFilter;


our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# fake section to be processed in &convert_font_pkg
# a method to protect multisection macro from killing as the part of %clean section
$RPM::Source::SpecSection::SECTION_MACROS{'_font_pkg'}=1;

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora Font dependencies',
    -description => 'convert Fedora specific Font dependencies',
    -class => 'Fedora2ALT::DependencyFilter::Font',
    -tags => SPEC_TAG_BUILD_REQUIRES | SPEC_TAG_BUILD_CONFLICTS | SPEC_TAG_REQUIRES | SPEC_TAG_CONFLICTS | SPEC_TAG_PROVIDES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora Specific',
    -description => 'fedora specific dependency filter',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraSpecific',
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora MimeHandler Deps',
    -description => 'convert Fedora MimeHandler dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraMimeHandlerDeps',
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-enable_fedora_font_posthook => 0,
	});
}


sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'fedora2alt_preconvert_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_font_pkg_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'fedora2alt_preconvert_generic',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'fedora2alt_convert',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_convert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POSTHOOKS,
	    # TODO: insert_first => 1 ? it seems there is no need
	    -name => 'fedora2alt_posthook_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_font_pkg_posthook);
}

my $FEDORA_VERSION=21;

# before DistroMap::depencencies
sub opt_handler_fedora2alt_preconvert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    # fedora perl packaging
	    #Requires:       perl(:MODULE_COMPAT_%(eval "`%{__perl} -V:version`"; echo $version))
	    #Requires:       perl(:MODULE_COMPAT_%(eval $(%{__perl} -V:version); echo $version))
	    #Requires:       perl(:MODULE_COMPAT_%{perl_version}.pm)
	    $sec->map_body(sub { 
		s/^(?:Build)?Requires:\s*$// if s/perl\(:MODULE_COMPAT_\%\(eval.+\)\)// or
		    s/perl\(:MODULE_COMPAT_\%\{?perl_version\}?(?:\.pm)?\)//;
		# ALT have no such macros :(
		s/(?<!%)\%{sparc}//g if /(?:Exclusive|Exclude|Build)Arch/ or /\%ifarch/;
		# nodejs support
		$_='' if /ExclusiveArch:\s+\%{nodejs_arches}/;
		s,\%{\??nodejs_find_provides_and_requires},,;
			   } );
	}
    }
}


sub opt_handler_fedora2alt_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;
    $mainsec->subst_body(qr'\%{x86}','%{ix86}',qr'^ExclusiveArch:');

    my ($flag_macro_fedoraver, $flag_macro_isa_bits, $flag_macro_texmf);
    my ($flag_macro_bcond_with, $flag_need_fedora_compat, $flag_need_cmake);
    my ($flag_macro_python2,$flag_macro_python3,$flag_macro_ruby, $flag_macro_kde,
	$flag_macro_erlang, $flag_macro_nodejs, $flag_qt4_devel, $flag_macro_power64);
    my ($filter_from_provreq_is_used,$configure_macro_is_used);
    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    #$sec->map_body(sub { } );

	    # group uncovering; tmp code till conditional macros handling will be improved
	    #%if 0%{?rhel}
	    #Group: Development/Languages
	    #%endif
	    my (@new_body,@tmp_storage);
	    foreach my $line (@{$sec->get_bodyref()}) {
		if ($line=~/^\s*\%if\s+0\%{\?rhel}/) {
		    push @tmp_storage, $line;
		} elsif (@tmp_storage) {
		    if ($line=~/^\s*Group:/) {
			push @new_body, $line, @tmp_storage;
		    } else {
			push @new_body, @tmp_storage, $line;
		    }
		    @tmp_storage=();
		} else {
		    push @new_body, $line;
		}
	    }
	    $sec->set_body(\@new_body);
	} elsif ($t eq 'build') {
	    $sec->map_body(sub {
		s!(?:\.|source)\s+(?:/etc|\%{_sysconfdir})/profile.d/qt.sh!. /etc/profile.d/qt3dir.sh; export PATH=%_libdir/qt3/bin:\$PATH!;
		$flag_qt4_devel=1 if s,\%{?_qt4_qmake}?,qmake-qt4,;
		$flag_qt4_devel=1 if /qmake-qt4/;
			   });
	    if ($sec->match_body('^\s*qmake(?:\s|$)')) {
		my $qtver=3;
		$qtver=4 if $mainsec->match_body('^BuildReq.+qt4-devel');
		$sec->subst_body(qr'^\s*qmake(?=(?:\s|$))','qmake-qt'.$qtver);
	    }
	} elsif ($t eq 'install') {
	    my $desktop_install=0;
	    $sec->map_body(sub {
		if (m!\.ttf!) {
		    # TnL-data, for example
		    s,/fonts/dejavu/,/fonts/ttf/dejavu/,;
		    # manaplus, for example
		    s,/fonts/liberation/,/fonts/ttf/liberation/,;
		    s,/fonts/mplus/,/fonts/ttf/mplus/,;
		    # python-pgu
		    s,/fonts/bitstream-vera/,/fonts/ttf/TrueType-vera/,;
		}
		# exclude
		$_='# '.$_ if /^\%{_fixperms}/;
		#ldconfig -v -n  %{buildroot}%{_libdir}
		$_='# '.$_ if /^ldconfig\s.*\%{_libdir}/;

		# desktop-file-install
		if (/^\s*desktop-file-install\s/ or ($desktop_install and /\\\n$/s)) {
		    $desktop_install=1;
		    $_='' if /^\s*--add-category\s+X-Fedora\s*\\\n/s;
		    # TODO: vendor
		    # 1) -> replace --vendor=fedora with %add_desktop_vendor
		    # 2) drop fedora-* in source / files / or detect applications/*
		    # if 2) fails, define add_desktop_vendor --vendor=fedora
		} else {
		    $desktop_install=0;
		}

			   });

	} elsif ($t eq 'triggerun') {
	    if ($sec->match_body(qr'(?:chkconfig|systemctl)')) {
		$sec->delete;
		next;
	    }

	} elsif ($t eq 'post') {
	$sec->map_body(sub {
	    $flag_need_fedora_compat=1 if /%systemd_post/;
		       } );

	} elsif ($t eq 'files') {
	    #
	}

	$sec->map_body(sub {
	    s,\%{_configure},%{_configure_script},g; # found in RuFedora monodevel-java
	    if (s/(?<!%)\%{?cmake}?(?=\s|$)/%{fedora_cmake}/g) {
		$flag_need_fedora_compat=1;
		$flag_need_cmake=1;
	    }
	    $flag_need_cmake=1 if s,\%{__cmake},cmake,g;
	    $configure_macro_is_used=1 if /\%{?configure/;

	    #cp -f /usr/lib/rpm/config.{guess,sub} config/
	    $_='# '.$_ if m!^\s*cp -[fp] /usr/lib/rpm/config\.!;
	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep');


	# general hacks here
	$sec->map_body(sub {

	    $flag_macro_power64=1 if /\%{power64}/;

	    # dislocations
	    s,\%{_sbindir}/strfile,%{_bindir}/strfile,g;

	    # translate macros first, as they are used in complex constructs
	    if (/(?<!%)\%{?python2?_site(?:lib|arch)}?(?!dir)/) {
		s,\%{python2?_sitelib},%{python_sitelibdir_noarch},g;
		s,\%python2?_sitelib,%python_sitelibdir_noarch,g;
		s,\%{python2?_sitearch},%{python_sitelibdir},g;
		s,\%python2?_sitearch,%python_sitelibdir,g;
		$flag_macro_python2=1;
	    }
	    if (/(?<!%)\%{?__python2}?/) {
		s,\%{__python2},%{__python},g;
		s,\%__python2(\s),%{__python}$1,g;
		$flag_macro_python2=1;
	    }

	    if (/(?<!%)\%{?python3_site(?:lib|arch)}?(?!dir)/) {
		s,\%{python3_sitelib},%{python3_sitelibdir_noarch},g;
		s,\%python3_sitelib,%python3_sitelibdir_noarch,g;
		s,\%{python3_sitearch},%{python3_sitelibdir},g;
		s,\%python3_sitearch,%python3_sitelibdir,g;
		$flag_macro_python3=1;
	    }
	    if (
		s,\%{py3dir},%{_builddir}/python3-%{name}-%{version}-%{release},g or
		s,\%py3dir,%{_builddir}/python3-%{name}-%{version}-%{release},g) {
		$flag_macro_python3=1;
	    }

	    if (/\%{?ruby_vendor(?:arch|lib)dir}?/) {
		s,\%{ruby_vendorlibdir},%{ruby_sitelibdir},g;
		s,\%ruby_vendorlibdir,%ruby_sitelibdir,g;
		s,\%{ruby_vendorarchdir},%{ruby_sitearchdir},g;
		s,\%ruby_vendorarchdir,%ruby_sitearchdir,g;
		$flag_macro_ruby=1;
	    }

	    if (/(?<!%)\%{?_texmf_main}?/) {
		s,\%{_texmf_main},%{_texmfmain},g;
		s,\%_texmf_main,%_texmfmain,g;
		$flag_macro_texmf=1;
	    }

	    if (/(?<!%)\%{?_kde4_(?:prefix|iconsdir|sbindir|bindir)}?/) {
		s,\%{_kde4_prefix},%{_K4prefix},g;
		s,\%_kde4_prefix,%_K4prefix,g;
		s,\%{_kde4_iconsdir},%{_K4iconsdir},g;
		s,\%_kde4_iconsdir,%_K4iconsdir,g;
		s,\%{_kde4_sbindir},%{_K4sbindir},g;
		s,\%_kde4_sbindir,%_K4sbindir,g;
		s,\%{_kde4_bindir},%{_K4bindir},g;
		s,\%_kde4_bindir,%_K4bindir,g;
		$flag_need_fedora_compat=1;
		$flag_macro_kde=1;
	    }

	    $flag_macro_erlang=1 if s,\%erlangdir,%_otpdir,g;
	    $flag_macro_erlang=1 if s,\%{erlangdir},%{_otpdir},g;
	    $flag_macro_fedoraver=1 if m'(?<!%)\%{\??fedora}';
	    $flag_macro_fedoraver=1 if m'(?<!%)\%\??fedora(?:[\s=<>]|$)';
	    $flag_macro_isa_bits=1 if m'(?<!%)\%{__isa_bits}';
	    $flag_macro_bcond_with=1 if m'(?<!%)\%bcond_with';
	    $flag_macro_nodejs=1 if m'(?<!%)\%{?nodejs_site(?:lib|arch)';

	    $flag_need_fedora_compat=1 if (
		m'(?<!%)\%{?(?:ocaml|mono|ghc|GNAT)_arches}?' or
		# qt4 compat macroses
		m'(?<!%)\%{?_qt4[}_]' or
		# perl compat
		m'(?<!%)\%{?fix_shbang_line}?' or
		# kde4 compat
		m'(?<!%)\%{?(?:cmake_kde4|_kde4_[\w\d_]+)}?' or
		# python 3 compat
		m'(?<!%)\%{?__python3}?' or
		# sugar
		m'(?<!%)\%{?sugar(?:activity|lib)dir}?' or
		# misc
		m'(?<!%)\%{?_(?:datarootdir|_id_u)}?(?=(?:\W|$))' or
		m'(?<!%)\%{?_cups_serverbin}?'
	    );

	    s,(?<!%)\%{_localstatedir}/,%{_var}/,g;
	    # alternatively,...
	    # altlinux vs redhat incompatibility (found in hsqldb)
	    #$sec->subst_body(qr'%{_localstatedir}/lib', '%{_var}/lib');
	    #$sec->subst_body(qr'%{_localstatedir}/log', '%{_var}/log');# jetty 5
	    #$sec->subst_body(qr'%{_localstatedir}/run', '%{_var}/run');# jetty 5
	    #$sec->subst_body(qr'%{_localstatedir}/cache', '%{_var}/cache');# jetty 5
	    s,\%{_emacs_sitelispdir},%{_emacslispdir},g;
	    s,\%_emacs_sitelispdir,%_emacslispdir,g;
	    s,\%{_emacs_sitestartdir},%{_emacs_sitestart_dir},g;
	    s,\%_emacs_sitestartdir,%_emacs_sitestart_dir,g;
	    s,\%{_emacs_bytecompile},%{byte_compile_file},g;
	    s,\%_emacs_bytecompile,%byte_compile_file,g;

	    s,\%{_xemacs_sitelispdir},%{_xemacs_lispdir},g;
	    s,\%_xemacs_sitelispdir,%_xemacs_lispdir,g;
	    s,\%{_xemacs_bytecompile},%{xemacs_bytecompile},g;
	    s,\%_xemacs_bytecompile,%xemacs_bytecompile,g;

	    s,\%{_tmppath},%{_var}/tmp,g;
	    s,\%{python_version},%{__python_version},g if not $macros->get_raw('python_version');
	    s,\%{_systemd_unitdir},%{_unitdir},g;

	    # translate complex constructs last

	    return if s,^\%{\!?\??(?:perl_default_filter|filter_setup):(?=\s*$),%{echo ,;
	    return if s,^\%if\s*\%{\!?\??(?:perl_default_filter|filter_setup)\b,%if 0%{echo,;
	    # brackets {} balance is important, as closing bracket for %{echo above can be %?perl_default_filter}
	    s,(?<!%)\%\{\!?\??(?:perl_default_filter|filter_setup)\},,;
	    s,(?<!%)\%\!?\??(?:perl_default_filter|filter_setup)\b,,;

	    # kill fancy fedora hacks that may disrupt alt mashinery
	    s/^\%(global\s+(?:_use_internal_dependency_generator|__deploop|__find_provides|__find_requires))/#$1/;
	    s/^(\s*chmod\s+\S+\s+\%\{?(?:__find_provides|__find_requires))/#$1/;
	    if (/^\%filter_from_(?:provides|requires)/) {
		s!::!.!g if /perl\(/;
		s!perl\(([^\)]+)\)!perl.$1.pm.!g;
		# and glue it together like in %filter_from_requires /perl(...)$/d /perl(...)$/d
		s!/d\s+/!/d;/!g;
		$filter_from_provreq_is_used=1;
	    } elsif (s/^\%global\s+__(provides|requires)_exclude\s+(.*)$//) {
		if ($filter_from_provreq_is_used) {
		    # entirely skip the conversion (old rpm 4.9) in favor of filter_from_...
		} else {
		    #%global __provides_exclude %{?__provides_exclude}|perl\\(Experimental::Sample\\)|perl\\(fail_example\\)|perl\\(fail_example_testsuite_setup\\)
		    #%global __requires_exclude %{?__requires_exclude}|perl\\(Exporter\\)
		    #%global __requires_exclude %{?__requires_exclude:%__requires_exclude|}^perl\\((Encode|File::ShareDir|MIME::Words)\\)$
		    my $cmd='%filter_from_'.$1;
		    my $opts=$2;
		    my $arg='';
		    $opts=~s/\%{?\??__(?:provides|requires)_exclude(?::\%\{?__(?:provides|requires)_exclude\}?)?\|?}?\|?//g;
		    if ($opts=~/perl\\\\\(/) {
			$opts=~s!::!/!g;
			$opts=~s!\\\\\)!.pm\\\\)!g;
			#$opts=~s/perl\\\\\(([\w_:]+)\\\\\)/&__perldep2exclpttern($1)/eg;
		    }
		    $opts=~s!/!.!g;
		    push @__push_mainsec, $cmd.' /'.$opts.'/d'."\n";
		}
	    } elsif (s/^\%filter_(prov|req)(?:ides|uires)_in\s+(.*)$//) {
		my $type=$1;
		my $path=$2;
		$path=~s/\s*$//;
		$path=~s!/$!/*!;
		push @__push_mainsec, '%add_find'.$type.'_skiplist '.$path."\n";
	    }

		       } );

    }

    unless ($configure_macro_is_used) {
	foreach my $sec ($spec->get_sections()) {
	    next if $sec->get_type ne 'files';
	    # Oops! did not work in case of arptables_jf
	    #$sec->subst_body(qr'\%{?_libexecdir}?/','/usr/libexec/');
	}
    }

    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-macros-fedora-compat') if $flag_need_fedora_compat;
    $buildreq->add_pre_buildreq('rpm-build-texmf') if $flag_macro_texmf;
    $buildreq->add_pre_buildreq('rpm-build-ruby') if $flag_macro_ruby;
    $buildreq->add_pre_buildreq('rpm-build-erlang') if $flag_macro_erlang;
    $buildreq->add_pre_buildreq('rpm-macros-kde-common-devel') if $flag_macro_kde;
    $buildreq->add_pre_buildreq('rpm-macros-nodejs') if $flag_macro_nodejs;
    if ($flag_macro_python2) {
	$buildreq->add_pre_buildreq('rpm-build-python');
	$buildreq->add_buildreq('python-devel');
    }
    if ($flag_macro_python3) {
	$buildreq->add_pre_buildreq('rpm-build-python3');
	$buildreq->add_buildreq('python3-devel');
    }
    $buildreq->add_buildreq('cmake') if $flag_need_cmake; # like kde apps ...
    $buildreq->add_buildreq('libqt4-devel') if $flag_qt4_devel;
    $mainsec->unshift_body('%define power64 ppc64'."\n") if $flag_macro_power64;
    $mainsec->unshift_body('%define fedora '.$FEDORA_VERSION."\n") if $flag_macro_fedoraver;
    $mainsec->unshift_body('# fedora __isa_bits tmp hack
%ifarch x86_64
%define __isa_bits 64
%else
%define __isa_bits 32
%endif
') if $flag_macro_isa_bits;
    $mainsec->unshift_body('# fedora bcond_with macro
%define bcond_with() %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}
# redefine altlinux specific with and without
%define with()         %{expand:%%{?with_%{1}:1}%%{!?with_%{1}:0}}
%define without()      %{expand:%%{?with_%{1}:0}%%{!?with_%{1}:1}}
') if $flag_macro_bcond_with;

    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_font_pkg_posthook {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK
	if not $opt->{-enable_fedora_font_posthook};

    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    if (not $spec->{'CONVERT::FEDORA::NO_MOVE_FONTS'}) {
		$sec->push_body(q!# generic fedora font import transformations
# move fonts to corresponding subdirs if any
for fontpatt in OTF TTF TTC otf ttf ttc pcf pcf.gz bdf afm pfa pfb; do
    case "$fontpatt" in 
	pcf*|bdf*) type=bitmap;;
	tt*|TT*) type=ttf;;
	otf|OTF) type=otf;;
	afm*|pf*) type=type1;;
    esac
    find $RPM_BUILD_ROOT/usr/share/fonts -type f -name '*.'$fontpatt | while read i; do
	j=`echo "$i" | sed -e s,/usr/share/fonts/,/usr/share/fonts/$type/,`;
	install -Dm644 "$i" "$j";
	rm -f "$i";
	olddir=`dirname "$i"`;
	mv -f "$olddir"/{encodings.dir,fonts.{dir,scale,alias}} `dirname "$j"`/ 2>/dev/null ||:
	rmdir -p "$olddir" 2>/dev/null ||:
    done
done
!);
	    }
	    if (not $spec->{'CONVERT::FEDORA::NO_RELINK_FONTCATALOG'}) {
		$sec->push_body(q!# kill invalid catalogue links
if [ -d $RPM_BUILD_ROOT/etc/X11/fontpath.d ]; then
    find -L $RPM_BUILD_ROOT/etc/X11/fontpath.d -type l -print -delete ||:
    # relink catalogue
    find $RPM_BUILD_ROOT/usr/share/fonts -name fonts.dir | while read i; do
	pri=10;
	j=`echo $i | sed -e s,$RPM_BUILD_ROOT/usr/share/fonts/,,`; type=${j%%%%/*}; 
	pre_stem=${j##$type/}; stem=`dirname $pre_stem|sed -e s,/,-,g`;
	case "$type" in 
	    bitmap) pri=10;;
	    ttf|ttf) pri=50;;
	    type1) pri=40;;
	esac
	ln -s /usr/share/fonts/$j $RPM_BUILD_ROOT/etc/X11/fontpath.d/"$stem:pri=$pri"
    done ||:
fi
!);
	    }
	} elsif ($t eq 'files') {
	    $sec->subst_body(qr'%{_fontdir}','%{_fontbasedir}/*/%{_fontstem}');
	    $sec->map_body(sub {
		if (m!^(.*)\%{catalogue}/(\S+)\s*$!) {
		    my $pathbase=$1;
		    my $stem=$2;
		    $stem=~s/-fonts$//;
		    $_=$pathbase.'%{catalogue}/'.$stem."*\n" unless /\*\s*$/;
		}
	    });
	}
    }
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_font_pkg_preconvert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    # first search for %_font_pkg
    my $flag_font_pkg_processed=0;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK unless $mainsec->match_body(qr'Build(?:Pre)?Req.*fontpackages-devel');
    my $font_type=&__get_font_type_from_name('',$distromap->unimap_source_origin2dest($spec->macros()->macro_subst('%name')));
    foreach my $sec (@sections) {
	$sec->apply_body(sub {
	    local $_=shift;
	    unless (/^\s*\%_font_pkg/) {
		return $_;
	    } else {
		#print STDERR "Yahoo! \%_font_pkg detected: $_";
		$flag_font_pkg_processed=1;
		chomp;
		s/^\s*\%_font_pkg\s*//;
		my @out='%files'."\n";
		my @filepattern;
		while (s/^(\S+)\s*//) {
		    if ($1 eq '-f') {
			s/^(\S+)\s*//;
			push @out, '%{_fontconfig_templatedir}/'."$1\n";
			push @out, '%config(noreplace) %{_fontconfig_confdir}/'."$1\n";
		    } elsif ($1 eq '-n') {
			s/^(\S+)\s*//;
			my $fontstem=$1;
			my $fcname=$fontstem;
			# -n can be a real package name as in ht-alegreya-fonts
			if (not $spec->get_section('package',"-n $fcname")) {
			    $fcname='%{?fontname:%fontname}%{!?fontname:%name}-'.$fontstem.'-fonts';
			    print STDERR "fedora2alt_preconvert_font_pkg: fontstem=$fontstem fcname=$fcname\n" if $verbose>1;
			    # mplus-fonts
			    $fcname=$fontstem.'-fonts' if ($fontstem =~/fontname/ and $fcname!~/-fonts$/);
			}
			my @mapname=$distromap->multimap_binary_origin2dest($spec->macros()->macro_subst($fcname));
			$font_type=&__get_font_type_from_name($font_type,@mapname);
			$mapname[0]||=$fcname;
			$out[0] ='%files -n '.$mapname[0]."\n";
		    } else {
			push @filepattern, $1;
		    }
		}
		s/\s*$//;
		push @filepattern, $_ if $_;
		# does not work for multi packages
		#push @out, '%{_fontbasedir}/*'."\n";
		push @out, map { '%{_fontbasedir}/*/%{_fontstem}/'.$_."\n"} @filepattern;
		return @out;
	    }
			 }, '%_font_pkg converter');
    }
    # not really needed; fontdir (not _fontdir) is used in few packages? baekamuk-bdf-fonts
    #$sections->push_body('%define fontdir %{_datadir}/fonts/'.$font_type.'/%{_fontstem}'."\n") if $font_type;
    $spec->_reset_speclist() if $flag_font_pkg_processed;
    $opt->{-enable_fedora_font_posthook}=1 if $flag_font_pkg_processed;
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub __get_font_type_from_name {
    my ($font_type,@mapname)=@_;
    if (@mapname and $mapname[0] =~ /^fonts-(ttf|otf|type1|bitmap)-/) {
	if ($font_type) {
	    $font_type='default' if $font_type ne $1;
	} else {
	    $font_type=$1;
	}
    }
    return $font_type;
}

1;

package RPM::Source::Tools::Dependency::Filter::FedoraSpecific;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    $dep->{-name} =~ s/\%\{?_isa}?$//;
    # tex dependencies conversion
    # proper way to do: but, due to tetex-latex, map to texmf-latex
    #$dep->{-name} =~ s,^tex\(([^\)]+)\.sty\)$,texmf(latex/$1),g;
    $dep->{-name}='texlive-latex-recommended' if $dep->{-name} =~ m,^tex\(([^\)]+)\.sty\)$,;
    return $dep;
}

package RPM::Source::Tools::Dependency::Filter::FedoraMimeHandlerDeps;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=1;
sub apply {
    my ($self, $tagobj, $dep) = @_;
    if ($dep->{-name} =~ /^mimehandler\(/) {
	print STDERR "info: dropped dependency $dep->{-name} - not yet supported in ALT \n" if $verbose;
	delete $dep->{-name};
    };
    return $dep;
}

package Fedora2ALT::DependencyFilter::Font;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^font\(:lang=(.+)\)$/;
    my $langreq=$1;
    $dep->{-name}='fontlang('.$langreq.')';
    return $dep;
}

1;

__END__
