package RPM::Source::Convert::Plugins::JPackage2ALT;

use strict;
use warnings;
use Carp;

use File::Basename;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Shared::Resource::DistroMap;
use RPM::Source::Convert::Plugins::Fedora2ALT;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $jvm=7;

my $TODO=qq~
%if %{native} %if ! %{gcj_support}
BuildRequires:  %{_bindir}/gcj, 
~;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'jpackage2alt_convert',
	    -opt_handler_sub=>\&opt_handler_jpp_specific_convert);
}

# TODO drop into distromap db
&init_plugin();
sub init_plugin {
    # BAD: if called after options, will overwrite option :( FIX in Groups.pm
    $RPM::Source::Convert::Plugins::DistroMap::Groups::default_group='Development/Java';

    map { $RPM::Source::Convert::Plugins::DistroMap::Packages::DEP_SKIP{$_}=1 }
# deprecated
qw!geronimo-specs-poms!, 
# java-devel jce jsse ...  are hack around Conflicts: in jpackage-1.x-compat
#jce jsse 
 qw!
java-devel java java-sdk java-headless
java-1.4.2-devel java-1.5.0-devel java-1.6.0-devel java-1.7.0-devel
java-1.4.2 java-1.5.0 java-1.6.0 java-1.7.0
jce jsse 
java-fonts!;

    map {$RPM::Source::Convert::Plugins::DistroMap::Groups::EXTRA_GROUP_PREMAP{$_} = 'Development/Java'} 
    (
	'Applications/Text',
	'Applications/Productivity',
	'Development',
	'Development/Build Tools',
	'Development/Code Generators',
	'Development/Compilers',
	'Development/Framework',
	'Development/Languages', # ecj
	'Development/Libraries',
	'Development/Libraries/Application Frameworks',
	'Development/Testing',
	'Development/Tools',
	'Development/C',
	'Development/Frameworks', # richfaces
	'Internet/WWW/Dynamic Content', #tomcat
	'Internet/WWW/Servers', #jboss
	'Internet/WWW/Indexing/Search', # lucene
	'Applications/Internet', # apacheds
	'System/Logging', # log4j
	'System/Networking', # mina
	'System Environment/Applications', #tomcat
	'System Environment/Libraries',
	'Security/Cryptography',
	'Software Development/Quality Assurance',
	'Networking/Daemons', # jacarta-common-daemons
	'System/Boot',
	'Utilities', # jarjar
	'Text Processing/Markup/XML',
	'Text Processing/Markup/HTML',
	'Documentation', # jacorb
	'Development Documentation', # rome
	'Development/Debuggers', # findbugs
    );
}

sub opt_handler_jpp_specific_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $main_sec=$spec->get_section('package','');
    my $name=$main_sec->get_tag('Name');
    $main_sec->unshift_body("BuildRequires: rpm-build-java-osgi\n") if $name=~/^eclipse/;

    if (not $main_sec->match_body(qr'BuildRequires: jpackage.*-compat')) {
	my $jppcompat='';
	if ($jvm) {
	    if ($jvm =~ /^4/) {
		$jppcompat='-1.4.2';
	    } elsif ($jvm =~ /^5/) {
		$jppcompat='-1.5.0';
	    } elsif ($jvm =~ /^6/) {
		$jppcompat='-1.6.0';
	    } elsif ($jvm =~ /^7/) {
		#$jppcompat='-1.6';
		# default
		$jppcompat='';
	    } elsif ($jvm =~ /^8/) {
		$jppcompat='-generic';
	    } else {
		$jppcompat='';
	    }
	}
	$main_sec->unshift_body("BuildRequires: jpackage$jppcompat-compat\n");
	$main_sec->unshift_body("BuildRequires: /proc\n");
    }

    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	#print "type is $t\n";
	if ($t eq 'package') {
	    &jpp_specific_find_bcond($sec) if $sec->is_main_section;
	    $sec->exclude_body(qr/^\s*\%define\s+section\s+free\s*$/);

	    $sec->subst_body(
		qr'\%define gcj_support \%{\?_with_gcj_support:1}\%{!\?_with_gcj_support:\%{\?_without_gcj_support:0}\%{!\?_without_gcj_support:\%{\?_gcj_support:\%{_gcj_support}}\%{!\?_gcj_support:0}}}', '%define gcj_support 0');
	    $sec->subst_body(qr'\%define\s+gcj_support\s+1', '%define gcj_support 0');
	    $sec->subst_body(qr'\%define\s+_with_gcj_support\s+1', '%define _with_gcj_support 0');

	    my $packagename=$sec->get_canonical_package();
	    # optimization
	    if ($packagename eq 'javadoc' || $packagename eq 'manual') {
		$sec->push_body('BuildArch: noarch'."\n") unless $sec->match_body(qr'^BuildArch:\s+noarch');
	    }
	} elsif ($t eq 'files') {
	    &jpp_specific_cleanup_files($sec);
	} elsif ($t eq 'prep' or $t eq 'build' or $t eq 'install') {
	    foreach my $java5pattern (
		qr'export JAVA_HOME=\%{_jvmdir}/java-1.5.0',
		qr'^(?:export\s+)?JAVA_HOME=/usr/lib/jvm/java-1.5.0',
		) {
		if ($sec->match_body($java5pattern)) {
		$sec->exclude_body($java5pattern);
		warn "excluded: export JAVA_HOME=/usr/lib/jvm/java-1.5.0\n";
		}
	    }
	} elsif ($t eq 'pre' or $t eq 'post' or $t eq 'preun' or $t eq 'postun') {
	    #
	}
    }
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub jpp_specific_find_bcond {
    my $section=shift;
    my @out;
    foreach (@{$section->get_bodyref()}) {
# at fault?: (jcommon.spec for example)
#%define bcond_with()    %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
#%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}
	
	if (/^\%bcond_with\s+(\S+)\s*$/) {
	    push @out, '#def_with '.$1."\n";
	    print STDERR 'JPackage2ALT: found disabled (bcond_with): '.$1."\n";
	}
	push @out, $_;
    }
    $section->set_body(\@out);
}


sub jpp_specific_cleanup_files {
    my $section=shift;
    my @out;
    foreach (@{$section->get_bodyref()}) {
	if (/^\s*\%ghost\s*((?:\%dir|\%doc)\s*)*\%{_javadocdir}/) {
	    # we will leave ghost there... a usual symlink will not upgrade from directory
#	    s,^\s*\%ghost\s*,,g;
	    push @out, $_;
#	} elsif (/^\s*\%ghost\s*\%dir\s*\%{_javadocdir}/) {
	    # throw away -- seems dangerous
#	} elsif (/^\s*\%ghost\s*%{_bindir}/) {
	} elsif (/^\s*\%ghost\s*\%{_javadir}/) {
	    # seems to be touch for alternatives. better to exclude.
	    s/^\s*\%ghost\s*/\%exclude /;
	    push @out, $_;
	} elsif (/^\s*\%ghost\s*/) {
	    # alternatives ?
	    push @out, "#".$_;
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

1;
