package RPM::Source::Convert::Plugins::SuSE2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Convert::Plugins::Generic2ALT;
use RPM::Source::Transformation::Factory::Buildreq;
use RPM::Source::Shared::Resource::DistroMap;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::default_originrepo='suse';
$DistroMap::default_originbranch=undef;

my $SUSE_VERSION=1150;

map {$RPM::Source::Convert::Plugins::DistroMap::Packages::DEP_SKIP{$_} = 1} # SUSE specific PreReq:
qw!%fillup_prereq %insserv_prereq!;

my %DEP_SUBST = (qw/update-desktop-files rpm-build-suse-compat/,
' ' => ' ',
);
foreach my $key (keys(%DEP_SUBST)) {
    $RPM::Source::Convert::Plugins::DistroMap::Packages::DEP_SUBST{$key} = $DEP_SUBST{$key};
}


# see also http://ru.opensuse.org/Сборка_пакетов/Соглашение_по_стилю_RPM-пакетов/Макросы_RPM

# TODO
#%build
#%{__make} %{?jobs:-j%jobs} %{?_smp_mflags}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'suse2alt_convert',
	    -opt_handler_sub=>\&opt_handler_suse2alt_convert);
}

sub opt_handler_suse2alt_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $mainsec=$spec->main_section;
    my ($flag_macro_suse,$flag_need_suse_compat);
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->exclude_body(qr'^\s*Autoreqprov: on');
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    $flag_need_suse_compat=1 if not $flag_need_suse_compat and $sec->match_body(qr'(?<!%)\%{?suse_update_desktop_file}?');
	    # /var/adm/fillup-templates -> /etc/sysconfig
	    $sec->subst_body(qr'/var/adm/fillup-templates(?=(?:\s|$))','/etc/sysconfig');
	    $sec->subst_body(qr'/var/adm/fillup-templates/sysconfig\.','/etc/sysconfig/');
	} elsif ($RPM::Source::SpecSection::SCRIPT_SECTION{$t}) {
	    $sec->exclude_body(qr'(?<!%)\%{?(?:insserv_cleanup|insserv_force_if_yast|fillup_only|fillup_and_insserv|install_info_delete|install_info|run_ldconfig)}?');
	} elsif ($t eq 'files') {
	    # /var/adm/fillup-templates -> /etc/sysconfig
	    $sec->subst_body(qr'/var/adm/fillup-templates/sysconfig\.','/etc/sysconfig/');
	    $sec->subst_body(qr'/var/adm/fillup-templates/\*','/etc/sysconfig/\*');
	}
	# general hacks here
	$flag_macro_suse=1 if not $flag_macro_suse and $sec->match_body(qr'(?<!%)\%{\??suse_version}');


    }
    $mainsec->unshift_body('%define suse_version '.$SUSE_VERSION."\n") if $flag_macro_suse;

    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-build-suse-compat') if $flag_need_suse_compat;
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

1;

__END__
